// 
// Copyright (C) 2007, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// Add this to your declarations to use StreamReader
using System.IO;

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// A quick sample demonstrating StreamReader properties. The techniques used in this indicator are also applicable to strategies.
    /// </summary>
    [Description("A quick sample demonstrating StreamReader properties.")]
    [Gui.Design.DisplayName("SampleStreamReader")]
    public class SampleStreamReader : Indicator
    {
        #region Variables
		private int		date				= 0;
		private double	currentOpen			= 0;
		private double	currentHigh			= 0;
		private double	currentLow			= 0;
		private double	currentClose		= 0;
		private bool	currentOpenSaved 	= false;
		private bool	currentLowSaved 	= false;
		
		// This sets the path in which the text file will be created.
		private string path 				= Cbi.Core.UserDataDir.ToString() + "MyTestFile.txt";
		
		// Creates a StreamReader object
		private System.IO.StreamReader sr;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose	= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {	
			// Prevents the NinjaScript from reading the file unless it is on a real-time bar
			if (Historical)
				return;
			
			// Checks to see if the file exists
			else if (File.Exists(path))
			{
				/* The try-catch block is used for error handling.
				In this case it is used to ensure you only have one stream object operating on the same file at any given moment. */
				try
				{
					// Sets the file the StreamReader will read from
					sr = new System.IO.StreamReader(path);
	
					string line;
					
					// Read lines and calculate the current day's OHLC from the file. While loop will go through the whole file.
					while ((line = sr.ReadLine()) != null) 
					{
						// Splits the line at every white space. Places each split into the string array
						string [] split = line.Split(new Char [] {' '});
						
						// Creates a counter that will mark which string from the split array we are dealing with
						int splitCounter = 0;
						
						// For every string inside the newly created string array...
						foreach (string s in split)
						{
							splitCounter++;
							// First string = Date of the OHLC data on the following 4 strings
							if (splitCounter == 1)
								date = int.Parse(s);
							
							// Check if the date of the data from file is the same as the date of the current bar
							if (ToDay(Time[0]) == date)
							{
								// Saves the first Open value of the current date
								if (splitCounter == 2 && currentOpenSaved == false)
								{
									currentOpen = double.Parse(s);
									
									// Unnecessary to save any other Open value's because they aren't the Open for the current date
									currentOpenSaved = true;
								}
								
								// Saves the High value of the current date. Overwrites old values if new value is higher
								else if (splitCounter == 3 && double.Parse(s) > currentHigh)
									currentHigh = double.Parse(s);
	
								// Saves the Low value of the current date
								else if (splitCounter == 4)
								{
									// Overwrites old values if new value is lower
									if (double.Parse(s) < currentLow)
										currentLow = double.Parse(s);
									
									// Initializes the first Low value because currentLow is defaulted at 0 and nothing will be lower than that
									else if (currentLowSaved == false)
									{
										currentLow = double.Parse(s);
										currentLowSaved = true;
									}
								}
								
								// Saves the Close value of the current date.
								else if (splitCounter == 5)
									currentClose = double.Parse(s);
							}
							
							// Resets the split counter for the next line read from the file
							if (splitCounter == 5)
								splitCounter = 0;
						}
					}
					Print("Current Day OHLC: " + currentOpen + " " + currentHigh + " " + currentLow + " " + currentClose);
				}
				
				catch (Exception e)
				{
					// Outputs the error to the log
					Log("You cannot write and read from the same file at the same time. Please remove SampleStreamWriter.", NinjaTrader.Cbi.LogLevel.Error);
					Print(e.ToString());
					throw;
				}
			}
			
			// If file does not exist, let the user know
			else
				Print("File does not exist.");
		}

		// Necessary to call in order to clean up resources used by the StreamReader object
		protected override void OnTermination()
		{
			// Disposes resources used by the StreamReader
			if (sr != null)
			{
				sr.Dispose();
				sr = null;
			}
		}
		
        #region Properties

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SampleStreamReader[] cacheSampleStreamReader = null;

        private static SampleStreamReader checkSampleStreamReader = new SampleStreamReader();

        /// <summary>
        /// A quick sample demonstrating StreamReader properties.
        /// </summary>
        /// <returns></returns>
        public SampleStreamReader SampleStreamReader()
        {
            return SampleStreamReader(Input);
        }

        /// <summary>
        /// A quick sample demonstrating StreamReader properties.
        /// </summary>
        /// <returns></returns>
        public SampleStreamReader SampleStreamReader(Data.IDataSeries input)
        {
            if (cacheSampleStreamReader != null)
                for (int idx = 0; idx < cacheSampleStreamReader.Length; idx++)
                    if (cacheSampleStreamReader[idx].EqualsInput(input))
                        return cacheSampleStreamReader[idx];

            lock (checkSampleStreamReader)
            {
                if (cacheSampleStreamReader != null)
                    for (int idx = 0; idx < cacheSampleStreamReader.Length; idx++)
                        if (cacheSampleStreamReader[idx].EqualsInput(input))
                            return cacheSampleStreamReader[idx];

                SampleStreamReader indicator = new SampleStreamReader();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                SampleStreamReader[] tmp = new SampleStreamReader[cacheSampleStreamReader == null ? 1 : cacheSampleStreamReader.Length + 1];
                if (cacheSampleStreamReader != null)
                    cacheSampleStreamReader.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSampleStreamReader = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// A quick sample demonstrating StreamReader properties.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleStreamReader SampleStreamReader()
        {
            return _indicator.SampleStreamReader(Input);
        }

        /// <summary>
        /// A quick sample demonstrating StreamReader properties.
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleStreamReader SampleStreamReader(Data.IDataSeries input)
        {
            return _indicator.SampleStreamReader(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// A quick sample demonstrating StreamReader properties.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleStreamReader SampleStreamReader()
        {
            return _indicator.SampleStreamReader(Input);
        }

        /// <summary>
        /// A quick sample demonstrating StreamReader properties.
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleStreamReader SampleStreamReader(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SampleStreamReader(input);
        }
    }
}
#endregion
